| omp_set_num_threads | R Documentation |
These functions get or set the maximal number of OpenMP threads that
can be used by knn_euclid and mst_euclid,
amongst others.
omp_set_num_threads(n_threads)
omp_get_max_threads()
n_threads |
maximal number of threads to use |
omp_get_max_threads returns the maximal number
of threads that will be used during the next call to a parallelised
function, not the maximal number of threads possibly available.
It there is no built-in support for OpenMP, 1 is always returned.
For omp_set_num_threads, the previous value of max_threads
is returned.
The official online manual of quitefastmst at https://quitefastmst.gagolewski.com/
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.